/**
 * Define api of apeman-api-db
 * @function api
 * @param {Apemandb} db - Instance of Apemandb class.
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict';

var co = require('co');
var aslogger = require('aslogger');

var describe = require('./describe');

/** @lends api */
function api(db) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var logger = aslogger({ disabled: !options.logging });

  return {
    ping: function ping() {
      var pong = arguments.length <= 0 || arguments[0] === undefined ? 'pong' : arguments[0];

      return co(regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                logger.trace('pong:', pong);
                return _context.abrupt('return', pong);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
    },
    setup: function setup(rootUsername, rootPassword) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee2() {
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return db.setup(rootUsername, rootPassword);

              case 2:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
    },
    sync: function sync() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var s = this;
      return co(regeneratorRuntime.mark(function _callee3() {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return db.sync(options);

              case 2:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
    },
    reset: function reset() {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee4() {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return db.reset();

              case 2:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));
    },
    describe: function describe() {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee5() {
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return db.describe();

              case 2:
                return _context5.abrupt('return', _context5.sent);

              case 3:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));
    },
    close: function close() {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee6() {
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                return _context6.abrupt('return', Promise.resolve(db.close()));

              case 1:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));
    },
    seed: function seed() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var s = this;
      return co(regeneratorRuntime.mark(function _callee7() {
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _context7.next = 2;
                return db.seed(options);

              case 2:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));
    },
    drop: function drop() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var s = this;
      return co(regeneratorRuntime.mark(function _callee8() {
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _context8.next = 2;
                return db.drop(options);

              case 2:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));
    },


    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before: function $before(methodName, params) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee9() {
        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                state.invokedAt = new Date();
                logger.info(methodName + ' started...');

              case 2:
              case 'end':
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after: function $after(methodName, params, returns) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee10() {
        var took;
        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                took = new Date() - state.invokedAt;

                logger.info('...' + methodName + ' done! (' + took + 'ms)');

              case 2:
              case 'end':
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));
    },

    /**
     * Method specifications
     */
    $spec: describe()
  };
}

module.exports = api;
//# sourceMappingURL=data:application/json;base64,bnVsbA==